
use raylib::{math::Vector2, prelude::*};
use crate::{player::Player, world::{chunk::CHUNK_WIDTH, id::BlockId, World}};

const SCREEN_BLOCK_WIDTH: f32 = 64.0;

pub struct Renderer {
    pub raylib: (RaylibHandle, RaylibThread),
}

impl Renderer {
    pub fn init() -> Self {
        let (rl, thread) = raylib::init()
            .vsync()
            .msaa_4x()
            .resizable()
            .size(640, 480)
            .title("Minecwaft :3")
            .log_level(raylib::ffi::TraceLogLevel::LOG_NONE)
            .build();
        Self { raylib: (rl, thread) }
    }

    pub fn should_close(&self) -> bool {
        self.raylib.0.window_should_close()
    }
    
    pub fn draw_world(&mut self, world: &World, player: &Player) {
        let (rl, thread) = &mut self.raylib;
        let scrn_wh: Vector2 = (rl.get_screen_width() as f32, rl.get_screen_height() as f32).into();

        let mut rl_draw = rl.begin_drawing(&thread);
        rl_draw.clear_background(Color::BLUE);
        
        let block_wh = scrn_wh.x as f32 / SCREEN_BLOCK_WIDTH;
        // let player_pos: Vector2;
        { // Player
            let scrn_mid: Vector2 = ((scrn_wh.x/2.0) as f32, (scrn_wh.y/2.0) as f32).into();
            // player_pos = (scrn_mid.x + block_wh/2.0, scrn_mid.y).into();
            let player_draw_pos: Vector2 = (scrn_mid.x + block_wh/2.0, scrn_mid.y + block_wh*2.0).into();

            let player_sz = Vector2::new(block_wh, block_wh*2.0);
            rl_draw.draw_rectangle_v(player_draw_pos, player_sz, Color::PINK);
        }

        { // World
            let block_sz = Vector2::new(block_wh, block_wh);
            
            for (chunk_x, chunk) in world.chunks() {
                for ((rel_x, block_y), block) in chunk.blocks() {
                    let block_x = (*chunk_x * CHUNK_WIDTH as isize) + *rel_x as isize;
                    
                    let block_y = (*block_y as f32 - 1.0) + player.pos.y;
                    let block_x = (block_x as f32) + player.pos.x;
                    
                    let real_x = block_x as f32 * block_wh;
                    let real_y = scrn_wh.y - (block_wh) - (block_y as f32 * block_wh);
                    
                    let color = match block.id {
                        BlockId::Dirt => Color::BROWN,
                        BlockId::Grass => Color::GREEN,
                        BlockId::Stone => Color::GRAY,
                        BlockId::Bedrock => Color::BLACK,
                    };
                    rl_draw.draw_rectangle_v(Vector2::new(real_x, real_y), block_sz, color);
                }
            }
        }

    }

    pub fn get_delta_time(&self) -> f32 {
        self.raylib.0.get_frame_time()
    }

}
